/*
{*NAME*}
	__FILE__ -- Routines for generating CDI mpeg1 ISO11172-1 stream

{*SYNOPSIS*}
	__FILE__

{*CLASS*}
	files:generic

{*DESCRIPTION*}
	This program is intended to create ISO11172-1 compliant system
	streams.  Its orginal incarnation was to create only CD-I compliant
	(video CD) streams, however, the parameter file now allows generic
	streams to also be created.  CD-I (Video CD) really only causes
	one problem in compliance and that is in the 20 byte audio padding.

{*ENVIRONMENT*}

{*SYSTEM REQUIREMENTS*}
	unix -- originally developed under both Sun 4.1.x and Linux 1.x

{*COMPILATION INSTRUCTIONS*}
	make binary

{*RESOURCE CONTROL*}
	$Id: cdisys.c,v 1.13 1994/12/15 02:12:47 quandt Exp quandt $
	$Log: cdisys.c,v $

{*SEE ALSO*}

{*BUGS*}
	EOF is not handled properly at this point

{*CREDITS*}
	HEURIS Logic -- Brian Quandt

{*COPYRIGHT*}
  	Copyright 1994 Heuris Logic.
  	All rights reserved.
{**}
*/


#include <stdio.h>
#include <unistd.h>

/* kludge code for dos, so that I have 32 bit ints to work with  */
/* #define int long */


#include <cdisys.h>
#include <cdisys.p>
#include <strstrn.p>
#include <main.p>

FILE *outfile;

/*
{*NAME*}
	main -- Main driver for the CDI MPEG1 ISO11172-1 system encoder

{*SYNOPSIS*}
	int main(int argc, char *argv[])

{*CLASS*}
	c:mpeg

{*DESCRIPTION*}
	Main routine for producing a mpeg system compliant stream.  All
	the input is taken from a parameter file.

{*ALGORITHM*}
	Get all the parameters 
	Produce a stream

{*SEE ALSO*}

{*BUGS*}
	Buffering mechanism needs more accuracy
	Dependent on video N/M relationship remaining fixed/constant

{*CREDITS*}
	HEURIS Logic -- Brian Quandt

{*COPYRIGHT*}
  	Copyright 1994 Heuris Logic.
  	All rights reserved.
{**}
*/
int main(int argc, char *argv[])
	{
	struct inparams UserParams; /* user supplied parameters */
	struct calcparams CalcParams;	/* calculated from the user params */


	if (GetParams(argc,argv,&UserParams, &CalcParams) != 0)
		{
		fprintf(stderr,"Error reading input parameters\n");
		exitroutine();
		}

	ProcessStreams(&UserParams, &CalcParams);

	/* orginally opened in ReadUserParams */
	fclose(outfile);
	}

/*
{*NAME*}
	GetParams -- CDI ISO11172-1 retrieve/calculate input parameters.

{*SYNOPSIS*}
	static int GetParams(int argc, char *argv[],
		struct inparams *UserParams,
		struct calcparams *CalcParams)

{*CLASS*}
	c:mpeg

{*DESCRIPTION*}
	Create or get all the necessary parameters used in this program.

{*ALGORITHM*}
	Get the user parameters.
	Calculate the internal variables from the supplied user parameters.

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}
	HEURIS Logic -- Brian Quandt

{*COPYRIGHT*}
  	Copyright 1994 Heuris Logic.
  	All rights reserved.
{**}
*/
static int GetParams(int argc, char *argv[],
	struct inparams *UserParams,
	struct calcparams *CalcParams)
	{
	int ret = 0;
	float tmp;

	ReadUserParams(argc, argv, UserParams,CalcParams);

	/* This is the frame rate/sec *90000 */
	/* Each PTS gets incremented by this amount, it is based on the frame rate */
	CalcParams->videoPTSconst = 
		(float)1/(float)UserParams->videoframesec * CLOCKFREQ;

	/* This is the audio PTS constant, it is based on the number of samples
	per frame (derived from the standard as 1152 bytes) */
	CalcParams->audioPTSconst = (float)AUDIOSAMPLESPERAU/(float)UserParams->audiosamplerate * CLOCKFREQ;


	/* Now the reorder delay necessary for video frames of type I/P.
	This is necessary since ordering of the frames in the encoded
	form is different from decoding order.  Since B frames are displayed
	as soon as they are recieved this value is actually the number of
	frames between successvie P/I frames * the rate per frame (in
	other words the M * 1/framerate * CLOCKFREQ) */
	CalcParams->videoPTSdelay  = 
		CalcParams->videoPTSconst * UserParams->video_M;

	/* the clock starts at 0 (easier on the math for other things) */
	CalcParams->SCR = 0;


	/* Set the audio and video decoder start times... note that 
	video starts one frame period before its PTS */

	CalcParams->videoPTS = UserParams->firstVideoPTS - CalcParams->videoPTSconst;
	CalcParams->audioPTS = UserParams->firstAudioPTS ;


	return(ret);
	}



/*
{*NAME*}
	ReadUserParams -- CDI ISO1172-1 encoder, read user provided parameters.

{*SYNOPSIS*}
	static int ReadUserParams(int argc, char *argv[],
		struct inparams *UserParams,
		struct calcparams *CalcParams)

{*CLASS*}
	c:mpeg

{*DESCRIPTION*}
	Read the user parameters from the supplied param file (command line option).
	A sample param file follows
 
		# Sample param file for system encoder
		audio.mpg 	# Name of the input audio file 
		video.mpg  	# Name of the input video file
		sys.mpg			# Name of the output system file
		e0				# hex value for video id, normally e0 (dec 224)
		c0				# hex value for audio id, normally c0 (dec 192)
		3				# MPEG video M value, ie size between P frames
		1152000.0		# Video bit rate
		224000.0		# Audio bit rate
		0				# bool flag, encoder continues even on EOF of aud/vid
		500				# max number of sectors to create
		3528			# muxrate, can be different from totaldeliveryrate/50
		15000			# First Audio pts in 90khz clock units
		21000			# First Video pts in 90khz clock units
		0				# Fixed bitrate flag 1->set, 0->variable bitrate stream
		1				# CSPS flag
		1				# Audio lock flag 1 or 0
		1				# Video lock flag 1 or 0
		44100.0			# Audio sample rate
		29.97			# Video frame rate
		2324			# Number of bytes per pack (CDI = 2324)
		2352			# Num bytes pack+any DSM related stuff (CDI = 2352)
		2312			# Number of bytes per video packet (CDI=2312)
		2294			# Number of data bytes per video packet (CDI=2294)
		2312			# Number of bytes per audio packet (CDI=2312)
		2279			# Number of data bytes per audio packet 
						(CDI=2279, don't include the 20 bytes of nulls here!)
		1				# Number of packets per pack (not implemented, fixed 
							to 1, CDI=1)
		1				# (0|1) If set then output CDI type headers and audio 
							(20byte) padding
		1				# (0|1) If set then include VCD type nulls with 
							audio packs

{*ALGORITHM*}
	Read all the parameters into the UserParam structure.

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}
	HEURIS Logic -- Brian Quandt

{*COPYRIGHT*}
  	Copyright 1994 Heuris Logic.
  	All rights reserved.
{**}
*/
static int ReadUserParams(int argc, char *argv[],
	struct inparams *UserParams, struct calcparams *CalcParams)
	{
	char	line[200];
	char 	stmp[100];
	char 	ctmp;
	int	itmp;
	float 	ftmp;
	FILE *paramfil;


	/* the first param is the name of the param file */
	if ((argc != 2)||(access(argv[1],R_OK)))
		{
		fprintf(stderr,"usage: cdisys paramfile\n");
		exit(-1);
		}
	
	
	if ( (paramfil = fopen(argv[1],"r")) == NULL)
		{
		fprintf(stderr,"could not open param file?\n");
		exit(-1);
		}

	/* skip the first line */
	fgets(line,MAXLINEBUF,paramfil);

	/* read the audio file name */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%s",stmp);
	UserParams->audio = fopen(stmp,"rb");
	/* Open an additional file handle into the stream for traversing AU's */
	UserParams->audioAU = fopen(stmp,"rb");

	/* read the video file name */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%s",stmp);
	UserParams->video = fopen(stmp,"rb");
	/* Open an additional file handle into the stream for traversing AU's */
	UserParams->videoAU = fopen(stmp,"rb");

	/* read the output system file name */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%s",stmp);
	UserParams->outfile = fopen(stmp,"wb");

	/* read the value for the video id */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%x",&itmp);
	UserParams->videostreamid = (unsigned char)itmp;

	/* read the value for the audio id */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%x",&itmp);
	UserParams->audiostreamid = (unsigned char)itmp;

	/* read the M value for distance to P frames */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->video_M = itmp;	/* number of frames between successive 

	/* read teh video bit rate */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->videorate =  ftmp;

	/* read the audio bit rate */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->audiorate = ftmp;

	/* read bool which determines how the encoder functinos on EOF */
	/* if set the encoder never stops, useless right... well not */
	/* if this ever gets put into a broadcast environment */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->runforever = itmp;

	/* read max number of sectors to read */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->maxsectors = itmp;

	/*read the mux rate, for now this can differ from the system bit rate */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->muxrate = itmp;

	/* Read in start time for first audio PTS */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->firstAudioPTS = itmp;
	UserParams->lastaudioSCR = itmp;

	/* Read in start time for first video PTS */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->firstVideoPTS = itmp;
	UserParams->lastvideoSCR = itmp;

	/* Read Fixed_flag ie fixed bitrate or variable bit rate */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->fixed_flag = itmp;

	/* Read CSPS_flag */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->CSPS_flag = itmp;

	/* Read audio lock flag */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->audio_lock = itmp;

	/* Read video lock flag */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->video_lock = itmp;

	/* Read audio sample rate */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->audiosamplerate = ftmp; /* 44100.0; */

	/* Read video frames a second */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->videoframesec = ftmp; /* 29.97	number of video frames a second */

	/* pack size in bytes */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->packsize = ftmp; 

	/* DSM pack size in bytes */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->dsmpacksize = ftmp; 

	/* video packet size in bytes */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->vidpacketsize = ftmp; 

	/* video data packet size in bytes */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->viddatasize = ftmp; 

	/* audio packet size in bytes */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->audpacketsize = ftmp; 

	/* audio data bytes per packet */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%f",&ftmp);
	UserParams->auddatasize = ftmp; 

	/* Read packets per pack */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->packetsperpack = itmp; 

	/* Read the CDI flag, if set then output CDI type system streams */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->CDI_type = itmp;

	/* Read the param that determines if to pad audio packs with 20 nulls */
	/* this is a Video CD thing */
	fgets(line,MAXLINEBUF,paramfil);sscanf(line,"%d",&itmp);
	UserParams->VCD_nulls = itmp;

	fclose(paramfil);
	}


