/*
{*NAME*}
	__FILE__ -- Routines for outputing bits.

{*SYNOPSIS*}
	__FILE__

{*CLASS*}
	files:generic

{*DESCRIPTION*}
	These are a collection of utilities for sending out bits.  The 
	idea is to send a word/byte/string and have this software
	worry about actually putting the bits out.

	There are also marker routines that allow you to keep count
	of how many bits thus far sent, with the ability to zero the
	user defined counters.

{*ENVIRONMENT*}
	None

{*SYSTEM REQUIREMENTS*}
	None

{*COMPILATION INSTRUCTIONS*}
	gcc -c __FILE__
	If DEBUG is defined then the outbits routine will output HEX text.

{*RESOURCE CONTROL*}
	$Id: outbits.c,v 1.3 1996/03/03 22:57:02 roy Beta2 $
	$Log: outbits.c,v $

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}
	Brian Quandt

{*COPYRIGHT*}
  	Copyright 1993 Heuris Logic.
  	All rights reserved.
{**}
*/

#include <outbits.p>
#include <stdio.h>
#include <bits.h>

/* var used by debug print statements */
char tas_outbit[256];

extern FILE *outfile;

/* these modules should proably go in to there own file */
static int bitcount=0;	/* this is total byte count of bytes sent out, cannot be altered (zeroed) */


/*
{*NAME*}
	outbits -- Routine to send out bits (one byte at a time).

{*SYNOPSIS*}
	int outbits(int val, 	\* If whichin is 0 then read thi int as input *\
			int whichin, 	\* controls input from val or ptr *\
			char *ptr, 		\* if whichin is !0 read this string as input *\
			int howmany, 	\* how many bits to read from the input *\
			int startbit)	\* starting bit to start reading from *\

{*CLASS*}
	c:bytes

{*DESCRIPTION*}

{*ENVIRONMENT*}

{*SYSTEM REQUIREMENTS*}

{*ALGORITHM*}

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}

{*COPYRIGHT*}
  	Copyright 1993 Heuris Logic.
  	All rights reserved.
{**}
*/
int outbits(int val, 	/* If whichin is 0 then read thi int as input */
		int whichin, 	/* controls input from val or ptr */
		char *ptr, 		/* if whichin is !0 read this string as input */
		int howmany, 	/* how many bits to read from the input */
		int startbit)	/* starting bit to start reading from */
	{
	/* If ptr is NULL then a 32 bit int was passed to this function
	howmany controls how many bits to read from this int, startbit
	determines the place in the int to start.

	whichin controls which input to use the 32bit int or the byte pointer

	For example
		a call with the following params
		(10,NULL,14,15)
		will take the number 10 as a 32 bit int 0x000A and start
		with bit number 15 and output 14 bits so the output is
		'00000000000010'

	If the number to output goes past the rightmost bit (2^0) then
	zero's are padded.  The same is true if you want to start to the
	left of the the left most (2^31).

	The char *ptr variable is used as an alternate way to send
	long 'strings' of things to send out.  If not NULL this
	takes precedence over the int 'val'. */


	int i,j;
	static char buf;
	static int bitct=0;	/* counter of how many currently in buffer */

	if ( whichin != 1)
		{
		i = 0;
		while (i<howmany)
			{
			while((bitct != 8)&&(i < howmany))
				{
				if (BITSET(val, (startbit-i)))
					SETBIT(buf,7-bitct);
				else
					UNSETBIT(buf,7-bitct);

				bitct++; i++;
				}
			if (bitct == 8)
				{
				/* output the byte */
#ifdef DEBUG
				sprintf(tas_outbit,"%2.2x ",0x00FF & buf);
				xuxa_log(tas_outbit);
#else
				fwrite(&buf,1,1,outfile);
#endif
				bitct = 0;
				}
			}

		}
	else
		{
#ifdef DEBUG
		sprintf(tas_outbit,"\n\t\tDATA %d bytes\n",howmany/8);
		xuxa_log(tas_outbit);
#else
		fwrite(ptr,(howmany/8),1,outfile);
#endif
		}

	/* Update our counters */
	bitcount = bitcount + howmany;	/* update the total bit count */

	}



/* routines for user counters */
/* The user must supply a place to hold temporary counts, the user
should not and does not need to do anything with this variable */

/* userbitcount is used interally to hold the state of the static
counter when markbits was first called */

/* WARNING these counters are 32 bit counters only, I have NOT coded them
to work around the 32 bit boundaries (this probably should be changed) */

/*
{*NAME*}
	markbits -- Return the current bit count (used to zero the counter).
	
{*SYNOPSIS*}
	int markbits(int *userbitcount ) \*  Read current bit count *\

{*CLASS*}
	c:bytes

{*DESCRIPTION*}

{*ENVIRONMENT*}

{*SYSTEM REQUIREMENTS*}

{*ALGORITHM*}

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}

{*COPYRIGHT*}
  	Copyright 1993 Heuris Logic.
  	All rights reserved.
{**}
*/
int markbits(int *userbitcount ) /*  Read current bit count */
	{
	/* Set the counter to current number of bits output */
	*userbitcount = bitcount;
	}

/*
{*NAME*}
	getUserbitcount -- Return the number of bits since a markbit call.

{*SYNOPSIS*}
	int getUserbitcount(
		int *userbitcount) \* return bit count since markbit call *\

{*CLASS*}
	c:bytes

{*DESCRIPTION*}

{*ENVIRONMENT*}

{*SYSTEM REQUIREMENTS*}

{*ALGORITHM*}

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}

{*COPYRIGHT*}
  	Copyright 1993 Heuris Logic.
  	All rights reserved.
{**}
*/
int getUserbitcount(int *userbitcount) /* return bit count since markbit call */
	{
	/* return the total number of bits sent to output since the
	last call to markbits */

	return (bitcount - *userbitcount);
	}

/*
{*NAME*}
	getbitcount -- Return total number of bits output since the beginning.

{*SYNOPSIS*}
	int getbitcount(void)

{*CLASS*}
	c:bytes

{*DESCRIPTION*}

{*ENVIRONMENT*}

{*SYSTEM REQUIREMENTS*}

{*ALGORITHM*}

{*SEE ALSO*}

{*BUGS*}

{*CREDITS*}

{*COPYRIGHT*}
  	Copyright 1993 Heuris Logic.
  	All rights reserved.
{**}
*/
int getbitcount(void)
	{
	/* Return total number of bits sent to output */
	return bitcount;
	}
